/**
 * @file mutex.h
 *
 * @author Jan Hadrava
 * @ingroup iopccommon
 */


#ifndef IOPCMUTEX_H
#define IOPCMUTEX_H

#include <pthread.h>
#include "exceptionBase.h"

namespace iopc {

/**
 * @brief Mutex synchronization primitive
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL Mutex {
public:
	Mutex()
		throw (IopcNoMemoryException, IopcRsrcLackException);
	~Mutex()
		throw ();
	/** @brief Locks mutex. */
	void lock()
		throw (IopcDeadlockException);
	/** @brief Unlocks mutex. */
	void unlock()
		throw (IopcIncorrectUsageException);
private:	/* hiding */
	Mutex(const Mutex &);
	Mutex &operator=(const Mutex &);
protected:
	pthread_mutex_t mutex;	/**< Mutex proper. */
};

}/*namespace iopc*/


#endif
