/**
 * @file timeStamp.h
 *
 * @ingroup iopccommon
 * @author Jan Hadrava
 */

#ifndef TIMESTAMP_H_
#define TIMESTAMP_H_

#include "defs.h"

#ifdef WIN32
#include <pthread.h>
#include <sys/timeb.h>
#else
#include <sys/time.h>
#endif /*WIN32*/

namespace iopc {

#ifdef WIN32

struct timeval {
	long tv_sec;
	long tv_usec;
};

#endif /*WIN32*/


typedef struct timespec TimeSpec;

/**
 * @brief A timeval wrapper. Represents a timestamp with microsecond accuracy.
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL TimeStamp {

public:
	TimeStamp();
	~TimeStamp();
	bool operator==(const TimeStamp& ts) const;
	bool operator!=(const TimeStamp& ts) const;
	bool operator<(const TimeStamp& ts) const;
	bool operator>(const TimeStamp& ts) const;
	bool operator<=(const TimeStamp& ts) const;
	bool operator>=(const TimeStamp& ts) const;
	long operator-(const TimeStamp& ts) const;
	TimeStamp operator+(long ms) const;
	operator TimeSpec() const;

	static TimeStamp now();

protected:
	void normalize();

	struct timeval tv;
};

}

#endif /*TIMESTAMP_H_*/
