/**
 * @file typeMappingFeature.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */

#ifndef TYPEMAPPINGFEATURE_H_
#define TYPEMAPPINGFEATURE_H_

#include "dbFeature.h"
#include "iopccommon/metadataHolder.h"
#include <vector>

namespace iopc {

class Type;

/**
 * @brief Declares methods that are used to translate C++ data types to SQL data types
 *
 * @ingroup iopcdb
 */
class TypeMappingFeature : public DbFeature
{
private:
	static std::string featureName;
public:
	virtual const std::string& getNameInst() {
		return getName();
	}

	static const std::string& getName() {
		return featureName;
	}

	/**
	 * @brief Returns a SQL column data type representation of the specified type
	 *
	 * Its format is equivalent to a column data type specification in a CREATE TABLE statement.
	 *
	 * Implementation should understand the [db.type.length] metadata passed either in the @a type or in the @a meta
	 * paramter to render proper column size for string data types.
	 *
	 * @param type The data type to be translated. Basic or enhanced data types can be used.
	 * @param meta Additional parameters.
	 */
	virtual std::string getTypeSql(const Type& type, const MetadataHolder& meta = MetadataHolder::emptyMeta) =0;
};


}


#endif /*TYPEMAPPINGFEATURE_H_*/
