/**
 * @file oracle10gSqlStatementsFeature.h
 *
 * @author Petr Cermak
 */

#ifndef ORACLE10GSQLSTATEMENTSFEATURE_H_
#define ORACLE10GSQLSTATEMENTSFEATURE_H_

#include "iopcdb/sqlStatementsFeature.h"

#include <vector>
#include <string>

namespace iopc {

class Oracle10gSqlStatementsFeature : public SqlStatementsFeature {

public:
	virtual std::string getCreateTableStatement(const MetadataHolder& tableMeta, const std::vector<const MetadataHolder*>& columns, const MetadataHolder& createMeta = MetadataHolder::emptyMeta);
	virtual std::string getDropTableStatement(const MetadataHolder& tableMeta, const MetadataHolder& dropmeta = MetadataHolder::emptyMeta);
	virtual std::string getInsertStatement(const MetadataHolder& tableMeta, const std::vector<const MetadataHolder*>& inColumns, const MetadataHolder& insertMeta = MetadataHolder::emptyMeta);
	virtual std::string getUpdateStatement(const MetadataHolder& tableMeta, const std::vector<const MetadataHolder*>& inColumns, const MetadataHolder& updateMeta = MetadataHolder::emptyMeta);
	virtual std::string getDeleteStatement(const MetadataHolder& tableMeta, const MetadataHolder& deleteMeta = MetadataHolder::emptyMeta);
	virtual std::string getSelectStatement(const std::vector<const MetadataHolder*>& tables, const std::vector<const MetadataHolder*>& outColumns, const MetadataHolder& selectMeta = MetadataHolder::emptyMeta);
	virtual std::string getCreateViewStatement(const MetadataHolder& meta, const MetadataHolder& createMeta = MetadataHolder::emptyMeta);
	virtual std::string getDropViewStatement(const MetadataHolder& meta, const MetadataHolder& dropMeta = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> getIndexes(const MetadataHolder& tableMeta, const std::vector<const MetadataHolder*>& columns);

	enum ColumnRenderingType {
		TABLE_COLS,
		VIEW_COLS,
		TABLEASVIEW_COLS
	};

	std::string getColumns(const MetadataHolder& tableMeta, const std::vector<const MetadataHolder*>& columns, bool doColumns, bool doConstraints, bool doTypes, bool typesAsParams = false, ColumnRenderingType columnRender = TABLE_COLS, std::string prefix = "");
	std::string getColumnsBindStr(const std::vector<const MetadataHolder*>& colMeta, bool doColNames = false, std::string prefix = "");
private:
	const MetadataHolder& getRootType(const MetadataHolder& type);
};


}

#endif /*ORACLE10GSQLSTATEMENTSFEATURE_H_*/
