/**
 * @file oracle10gObjectStatementsFeature.h
 *
 * @author Petr Cermak
 */

#ifndef ORACLE10GOBJECTSTATEMENTSFEATURE_H_
#define ORACLE10GOBJECTSTATEMENTSFEATURE_H_

#include "iopclib/db/objectStatementsFeature.h"
#include "iopcdriver_oracle10g/oracle10gSqlStatementsFeature.h"
#include "iopcdriver_oracle10g/oracle10gTypeMappingFeature.h"
#include "iopclib/types/dbType.h"
#include "iopclib/types/objectMappingStrategy.h"
#include "iopclib/db/statementCache.h"
#include <sstream>
#include <set>
#include <string>
#include <vector>
#include <sstream>
//#include <list>

namespace iopc {

class Type;

class Oracle10gObjectStatementsFeature : public ObjectStatementsFeature
{
private:
	Oracle10gSqlStatementsFeature* sqlStatements;
	Oracle10gTypeMappingFeature* typeMapping;

	std::vector<const MetadataHolder*> getColumnsInOrder(ObjectMappingStrategy& strategy);
	std::string getWhere(const DbType::KeyAttributesSet& keys, std::string metaColName);

	StatementCache& cache;
public:
	Oracle10gObjectStatementsFeature(StatementCache& cache) : cache(cache) {};

	inline void init(SqlStatementsFeature& sqlStatements,
		TypeMappingFeature& typeMapping) {
		this->sqlStatements = dynamic_cast<Oracle10gSqlStatementsFeature*>(&sqlStatements);
		this->typeMapping = dynamic_cast<Oracle10gTypeMappingFeature*>(&typeMapping);
	}

	virtual std::string getInsertStatement(ObjectMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getSelectFromClassViewStatement(ObjectMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getSelectFromClassTableStatement(ObjectMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getUpdateStatement(ObjectMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	//virtual std::string getUpdateStatement(ObjectMappingStrategy& currentStrategy, const ObjectMappingStrategy::MappedAttributesList& updateCols, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getDeleteStatement(ObjectMappingStrategy& currentStrategy, const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual bool validate(ObjectMappingStrategy& currentStrategy, std::string& msg);


	std::vector<std::string> getCreateClassTableScript(ObjectMappingStrategy& currentStrategy,  const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	std::vector<std::string> getDropClassTableScript(ObjectMappingStrategy& currentStrategy,  const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	void processAttributesForView(ObjectMappingStrategy& currentStrategy, bool& start, std::ostringstream& ocolsStr, std::set<std::string>& processedColumns, const Type& mainType);
};


}



#endif /*ORACLE10GOBJECTSTATEMENTSFEATURE_H_*/
