/**
 * @file fakeExtendedCache.h
 *
 * @author Jan Hadrava
 * @date 2004
 * @ingroup iopclibcache
 */

#ifndef FAKEEXTENDEDCACHE_H_
#define FAKEEXTENDEDCACHE_H_

// project: POLiTe
// authors: Jan Hadrava 2004


#include "cache.h"
#include "strategy.h"
#include "strategySelector.h"
#include "iopccommon/defs.h"

namespace iopc {

/** Makes basic cache look like an extended one.
 * @ingroup iopclibcache
 * */
class DLL_EXTERNAL FakeExtendedCache: public ExtendedCache {
public:
	inline FakeExtendedCache(Cache* cache, bool own = true);
	inline virtual ~FakeExtendedCache();
	inline virtual Cache* clone();
private:/*hiding*/
	FakeExtendedCache(FakeExtendedCache &);
	FakeExtendedCache &operator=(FakeExtendedCache &);
protected:
    virtual void setConnection(Connection* conn = NULL);
protected:
	inline virtual void clear();

	inline virtual CacheLock& makePersistent(const class PersIdentification& persId, DatabaseObject& objectPtr);
	inline virtual CacheLock& getCacheLock(const class PersIdentification& persId, bool rdonly = false);

	inline virtual void dbDelete(const PersIdentification& persId);
	inline virtual void cacheDelete(const PersIdentification& persId);

	inline virtual void updateDirty();
	inline virtual void removeAll();
protected:
	inline virtual void clear(const StrategySelector& strSel);

	inline virtual CacheLock& makePersistent(const StrategySelector& strSel, const PersIdentification& persId, DatabaseObject& objectPtr);
	inline virtual CacheLock& getCacheLock(const StrategySelector& strSel, const PersIdentification& persId, bool rdonly = false);

	inline virtual void dbDelete(const StrategySelector& strSel, const PersIdentification& persId);
	inline virtual void cacheDelete(const StrategySelector& strSel, const PersIdentification& persId);

	inline virtual void updateDirty(const StrategySelector& strSel);
	inline virtual void removeAll(const StrategySelector& strSel);
protected:
	Cache* cache;
	bool own;
};

inline Cache* FakeExtendedCache::clone()
{
	return new FakeExtendedCache(cache);
}

inline FakeExtendedCache::FakeExtendedCache(Cache* cache, bool own)
: cache(cache), own(own)
{
}

inline FakeExtendedCache::~FakeExtendedCache()
{
	if (own) delete cache;
}

inline void FakeExtendedCache::setConnection(Connection* conn)
{
	Cache::setConnection(conn);
	cache->setConnection(conn);
}


inline void FakeExtendedCache::clear()
{
	cache->clear();
}

inline CacheLock& FakeExtendedCache::makePersistent(const PersIdentification& persId, DatabaseObject& objectPtr)
{
	return cache->makePersistent(persId, objectPtr);
}

inline CacheLock& FakeExtendedCache::getCacheLock(const PersIdentification& persId, bool rdonly)
{
	return cache->getCacheLock(persId, rdonly);
}

inline void FakeExtendedCache::dbDelete(const PersIdentification& persId)
{
	cache->dbDelete(persId);
}

inline void FakeExtendedCache::cacheDelete(const PersIdentification& persId)
{
	cache->cacheDelete(persId);
}

inline void FakeExtendedCache::updateDirty()
{
	cache->updateDirty();
}

inline void FakeExtendedCache::removeAll()
{
	cache->removeAll();
}

inline void FakeExtendedCache::clear(const StrategySelector& strSel)
{
	cache->clear();
}

inline CacheLock& FakeExtendedCache::makePersistent(const StrategySelector& strSel, const PersIdentification& persId, DatabaseObject& objectPtr)
{
	return cache->makePersistent(persId, objectPtr);
}

inline CacheLock& FakeExtendedCache::getCacheLock(const StrategySelector& strSel, const PersIdentification& persId, bool rdonly)
{
	return cache->getCacheLock(persId, rdonly);
}

inline void FakeExtendedCache::dbDelete(const StrategySelector& strSel, const PersIdentification& persId)
{
	cache->dbDelete(persId);
}

inline void FakeExtendedCache::cacheDelete(const StrategySelector& strSel, const PersIdentification& persId)
{
	cache->cacheDelete(persId);
}

inline void FakeExtendedCache::updateDirty(const StrategySelector& strSel)
{
	cache->updateDirty();
}

inline void FakeExtendedCache::removeAll(const StrategySelector& strSel)
{
	cache->removeAll();
}

}/*namespace iopc*/


#endif /*FAKEEXTENDEDCACHE_H_*/
