/**
 * @file simpleCacheSelector.h
 *
 * @author Jan Hadrava
 * @date 2004
 * @ingroup iopclibcache
 */

#ifndef SIMPLECACHESELECTOR_H_
#define SIMPLECACHESELECTOR_H_

// project: POLiTe
// authors: Jan Hadrava 2004


#include <map>

#include "extendedCache.h"
#include "cacheSelector.h"
#include "../persIdentification.h"
#include "iopccommon/defs.h"

namespace iopc {

/** Simple implementation of cache selector.
 *
 * @ingroup iopclibcache
 * */
class DLL_EXTERNAL SimpleCacheSelector: public CacheSelector {
friend class CacheKeeper;
public:
	SimpleCacheSelector();
	virtual ~SimpleCacheSelector();
private:/*hiding*/
	SimpleCacheSelector(SimpleCacheSelector& );
	SimpleCacheSelector &operator=(SimpleCacheSelector& );
protected:
	typedef struct {
		ExtendedCache* cache;
		bool isMine;
	} CacheRecord;
	typedef std::map<const Type* , CacheRecord*> CacheMap;
	typedef CacheMap::iterator Iter;
public:
	ExtendedCache* firstCache();
	ExtendedCache* nextCache();
	ExtendedCache* selectCache(const PersIdentification& persId);
	/** Sets object cache for specified type. */
	void setCache(const Type& type, ExtendedCache* cache = NULL, bool isMine = false);
	/** Sets object cache for all types. */
	void setCache(ExtendedCache* cache = NULL, bool isMine = false);
protected:
	void setCacheInternal(const Type* type, ExtendedCache* cache = NULL, bool isMine = false);
	CacheMap cacheMap;
	Iter lastIter;
};

}/*namespace iopc*/


#endif /*SIMPLECACHESELECTOR_H_*/
