/**
 * @file XMLSupport.h
 *
 * @author Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>)
 * @date 2003 - 2004
 *
 * @brief Helper classes, functions and macros used when parsing / creating XML
 *        documents using Xerces XML parser.
 */

#ifndef XMLSUPPORT_H_
#define XMLSUPPORT_H_


//#include "XMLStrings.h"
//#include "messagePrinting.h"

#include <ostream>

//#include <xercesc/util/PlatformUtils.hpp>        //XMLPlatformUtils
#include <xercesc/util/XMLString.hpp>
//#include <xercesc/dom/DOM.hpp>                   //DOMException
//#include <xercesc/parsers/XercesDOMParser.hpp>   //XercesDOMParser
//#include <xercesc/sax/ErrorHandler.hpp>          //ErrorHandler
//#include <xercesc/sax/SAXParseException.hpp>     //SAXParseException
//#include <xercesc/validators/DTD/DTDGrammar.hpp> //DTDGrammar
//#include <xercesc/framework/LocalFileFormatTarget.hpp> //LocalFileFormatTarget


/*! \page licences Third party licences and acknowledgements

  \section xerces_licence Xerces-C licence

  XML Metadata Writer and XML Metadata Loader use Xerces XML parser. The licence
  of the parser follows:

\verbatim
The Apache Software License, Version 1.1


Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in
   the documentation and/or other materials provided with the
   distribution.

3. The end-user documentation included with the redistribution,
   if any, must include the following acknowledgment:
      "This product includes software developed by the
       Apache Software Foundation (http://www.apache.org/)."
   Alternately, this acknowledgment may appear in the software itself,
   if and wherever such third-party acknowledgments normally appear.

4. The names "Xerces" and "Apache Software Foundation" must
   not be used to endorse or promote products derived from this
   software without prior written permission. For written
   permission, please contact apache@apache.org.

5. Products derived from this software may not be called "Apache",
   nor may "Apache" appear in their name, without prior written
   permission of the Apache Software Foundation.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
====================================================================

This software consists of voluntary contributions made by many
individuals on behalf of the Apache Software Foundation and was
originally based on software copyright (c) 1999, International
Business Machines, Inc., http://www.ibm.com.  For more
information on the Apache Software Foundation, please see
<http://www.apache.org/>.
\endverbatim
*/

XERCES_CPP_NAMESPACE_USE

/*! Maximal size of buffer used when getting value of an attribute in getRequiredAttribute()
    or getOptionalAttribute().
 */
#define TAGNAME_BUFFER_SIZE 128

namespace iopc {

// ---------------------------------------------------------------------------
//! Simple class used to transcode given <tt>const XMLCh *</tt> to <tt>const char *</tt>.
class TranscodedString
{
public :
  //! Transcodes given string storing it in \ref localForm.
  TranscodedString(const XMLCh * src)
  {
    localForm = XMLString::transcode(src);
  }

  //! Simple destructor.
  ~TranscodedString()
  {
    XMLString::release(&localForm);
  }

  //! Returns transcoded string as <tt>const char *</tt>.
  operator const char *() const
  {
    return localForm;
  }

protected :
 //! Local code page form of given string.
  char * localForm;
};

/*! Operator needed to make it possible to send TranscodedString to <tt>ostream</tt> -
    without this operator TranscodedString would be converted to <tt>const char *</tt>,
    then to <tt>void *</tt> and printed as pointer.
 */
inline std::ostream & operator<<(std::ostream & target, const TranscodedString & toDump)
{
    target << (const char *)toDump;
    return target;
}

//! Macro to make using of TranscodedString easier (in Visual C++ useless).
#define TOSTRING(x) (const char *)TranscodedString(x)

//--------------------------------------------------------------------------------------
//! Simple class used to transcode given <tt>const char *</tt> to <tt>const XMLCh *</tt>.
class UnicodeString
{
public :
  //! Transcodes given string storing it in \ref unicodeForm.
  UnicodeString(const char * src)
  {
    unicodeForm = XMLString::transcode(src);
  }

  //! Simple destructor.
  ~UnicodeString()
  {
    XMLString::release(&unicodeForm);
  }

  //! Returns transcoded string as <tt>const XMLCh *</tt>.
  operator const XMLCh *() const
  {
    return unicodeForm;
  }

protected:
  //! Unicode form of given string.
  XMLCh * unicodeForm;
};

//! Macro to make using of UnicodeString easier (in Visual C++ useless)
#define TOXML(x) (const XMLCh *)UnicodeString(x)

//-----------------------------------------------------------------------------------


}
#endif /*XMLSUPPORT_H_*/
