/**
 * @file cachedDatabase.cpp
 *
 * @ingroup iopclib
 *
 * @author Petr Cermak
 */

#include "cachedDatabase.h"
#include "cachedConnection.h"

namespace iopc {

CachedDatabase::CachedDatabase(Database* database, Cache* cachePrototype)
{
	IOPC_FUNC_INVOKED;
	inner = database;
	cache = cachePrototype;
}

CachedDatabase::~CachedDatabase()
{
	IOPC_FUNC_INVOKED;
	delete cache;
	delete inner;
}

void CachedDatabase::commit()
{
	inner->commit();
}

void CachedDatabase::rollback()
{
	inner->rollback();
}

Connection* CachedDatabase::getConnection(const std::string& connStr)
{
	IOPC_FUNC_INVOKED;
	return new CachedConnection(inner->getConnection(connStr), cache->clone(), *this);
}

void CachedDatabase::returnConnection(Connection* connection)
{
	IOPC_FUNC_INVOKED;
	inner->returnConnection(connection);
	IOPC_FUNC_ENDED;
}

void CachedDatabase::disconnect()
{
	IOPC_FUNC_INVOKED;
	inner->disconnect();
}

DatabaseImpl* CachedDatabase::getImplementation()
{
	return inner->getImplementation();
}

Database* CachedDatabase::getBasicDatabase()
{
	return inner->getBasicDatabase();
}

Driver& CachedDatabase::getDriver()
{
	return inner->getDriver();
}

}
