#include <string>
#include <iostream>
#include <vector>

#include "iopcdb/iopcDbHeaders.h"

using namespace std;
using namespace iopc;

int main(int argc, char *argv[])
{
	Database* db = NULL;
	Connection* conn = NULL;
	try {
		// Uncomment the INFO_SQL mask to see what SQL statements are sent to the database
		IopcDb::init(LogWriter::ERROR | LogWriter::WARNING /*| LogWriter::INFO_SQL*/, LogWriter::NONE);

		// Create a database connection
		db = DriverManager::getDriver("IopcOracle10g").getDatabase("XE");
		conn = db->getConnection("username=iopc;password=iopc");
		conn->open();

		// Non-parametrised SQL statements
		try {
			conn->sqlNonQuery("DROP TABLE blacklist");
		} catch(...) {}
		conn->sqlNonQuery("CREATE TABLE blacklist(id NUMBER PRIMARY KEY, name VARCHAR2(20))");
		conn->sqlNonQuery("INSERT INTO blacklist VALUES(1, 'Richard Doe')");
		conn->sqlNonQuery("INSERT INTO blacklist VALUES(2, 'Ola Nordmann')");

		conn->commit();

		// Set the fetch buffer for strings to 100. Default is 2000
		//MetadataHolder meta;
		//meta["db.type.length"].setIntValue(100);

		// Select all rows from the blacklist table
		Cursor* cur = conn->sql("select id, name from blacklist");
		string name;
		int id;
		cur->addOutParam(1, &id, TypeDesc<int>::getType());
		cur->addOutParam(2, &name, TypeDesc<std::string>::getType());
		cout << endl << "Blacklist " << endl;
		cur->execute();
		while (cur->fetchNext()) {
			cout << id << " " << name << endl;
		}
		cur->close();
		conn->returnCursor(cur);

		// This time with enhanced data types
		cur = conn->sql("select id, name from blacklist");
		EString ename;
		EInt eid;
		// OUT parameters can be added any time before fetch is invoked
		// - even after execute
		cout << endl << "Blacklist (enhanced datatypes)" << endl;
		cur->execute();
		cur->addEnhancedOutParam(1, eid);
		cur->addEnhancedOutParam(2, ename);
		while (cur->fetchNext()) {
			cout << eid << " " << ename << endl;
		}
		cur->close();
		conn->returnCursor(cur);

		// Input parameters
		cur = conn->sql("SELECT name from blacklist where id = :id");
		eid = 2;
		cur->addEnhancedOutParam(1, ename); // first input parameter
		cur->addEnhancedInParam(1, eid); // first output parameter
		cout << endl << "Blacklist entry with id " << eid << endl;
		cur->execute();
		while (cur->fetchNext()) {
			cout << eid << " " << ename << endl;
		}
		cur->close();

		// Does the driver supports the SqlStatementsFeature?
		// If so, what SQL column type is associated with the string datatype?
		Driver& d = conn->getDriver();
		if (d.supportsFeature<SqlStatementsFeature>()) {
			TypeMappingFeature& f = d.getFeature<TypeMappingFeature>();
			string sqlType = f.getTypeSql(TypeDesc<string>::getType());
			cout << endl << "String is associated with: " << sqlType << endl;
		}

		conn->close();
		db->returnConnection(conn);
		DriverManager::getDriver("IopcOracle10g").returnDatabase(db);
		// alternatively:
		//DriverManager::returnDatabase(db);

		conn = NULL;
		db = NULL;

		IopcDb::shutdown();
	} catch (IopcException& e) {
		cout << e.what();
	}

	if (db && conn) conn->close();
	if (db) DriverManager::returnDatabase(db);

	cout << "-----------" << endl;
	cout << "Press enter" << endl;
	cin.getline(NULL, 0);
	return 0;
}


