#ifndef SCHEMA_H_
#define SCHEMA_H_

#include "iopclib/iopcHeaders.h"

using namespace iopc;
using namespace std;

class Person : public OidObject {
public:
	EShort age;
	EString name;
};

class Employee : public Person {
public:
	EInt salary;
};

class Student : public Person {
public:
	EString studcardid;
	DbPtr<Employee> supervisor;

	static void iopcInit(iopc::Type& t) {
		t["db.mapping.type"].setStringValue("horizontal");
	}
};
class PhdStudent : public Student {
public:
	EShort scholarship;

	static void iopcInit(iopc::Type& t) {
		t["db.mapping.type"].setStringValue("filtered");
		t["db.mapping.insertto"].setStringValue("::Student");
	}
};

class BlacklistEntry : public iopc::DatabaseObject {
public:
	EInt externalId;
	EString name;
	static void iopcInit(iopc::Type& t) {
		t["db.table"].setStringValue("blacklist");
		// No CREATE or DROP scripts will be created for this table.
		t["scripts.exclude"].setBoolValue(true);
		t.getAttribute("externalId")["db.column"].setStringValue("id");
		t.getAttribute("externalId")["db.primaryKey"].setBoolValue(true);
		t.getAttribute("name")["db.type.length"].setIntValue(20);
		// not necessary, attribute name and column name are the same:
		t.getAttribute("name")["db.column"].setStringValue("name");
	}
};

class UserTable : public iopc::DatabaseObject {
public:
	EString tableName;
	EString tablespaceName;

	static void iopcInit(iopc::Type& t) {
		t["db.transient"].setBoolValue(true);
		t["db.table"].setStringValue("USER_TABLES");
		t.getAttribute("tableName")["db.column"].setStringValue("TABLE_NAME");
		t.getAttribute("tableName")["db.type.length"].setIntValue(30);
		t.getAttribute("tablespaceName")["db.column"].setStringValue("TABLESPACE_NAME");
		t.getAttribute("tablespaceName")["db.type.length"].setIntValue(30);
	}
};

class UserTable2 : public iopc::DatabaseObject {
public:
	EString tableName;
	EString tablespaceName;
	// We added this attribute:
	EInt columnCount;

	static void iopcInit(iopc::Type& t) {
		t["db.transient"].setBoolValue(true);
		t["db.table"].setStringValue("USER_TABLES");
		t.getAttribute("tableName")["db.column"].setStringValue("TABLE_NAME");
		t.getAttribute("tableName")["db.type.length"].setIntValue(30);
		t.getAttribute("tablespaceName")["db.column"].setStringValue("TABLESPACE_NAME");
		t.getAttribute("tablespaceName")["db.type.length"].setIntValue(30);
		t.getAttribute("columnCount")["db.column.sql"].setStringValue("COUNT(*)");
		// Not necessary:
		t.getAttribute("columnCount")["db.column"].setStringValue("COLUMN_COUNT");
	}
};

#endif /*SCHEMA_H_*/
