/**
 * @file iopcCommon.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopccommon
 * @defgroup iopccommon iopccommon
 */

#ifndef IOPCCOMMON_H_
#define IOPCCOMMON_H_

#include "trace.h"

namespace iopc {

/**
 * @brief @c iopccommon library/configuration initialization class
 *
 * When using the @c iopccommon configuration, the library must be initialized
 * using the @a IopcCommon::init() method and terminated using the @a IopcCommon::shutdown() method.
 *
 * @ingroup iopccommon
 */
class IopcCommon {

public:
	/**
	 * @brief Initializes the @c iopccommon library/configuration
	 *
	 * @param traceMaskTTY @a LogWriter mask used for console output
	 * @param traceMask @a LogWriter mask used for log file output
	 * @param logFileName Name of the IOPC log file
	 */
	static void init(int traceMaskTTY = LogWriter::defaultTraceMaskTTY,
			int traceMask = LogWriter::defaultTraceMask,
			const std::string& logFileName = LogWriter::defaultFileName);

	/**
	 * @brief Terminates the @c iopccommon library/configuration
	 */
	static void shutdown();

//	static bool isInitialized(bool initialized = false);
};

}


#endif /*IOPCCOMMON_H_*/
