/**
 * @file rwLock.h
 *
 * @author Jan Hadrava
 * @ingroup iopccommon
 */

#ifndef RWLOCK_H
#define RWLOCK_H

#include <pthread.h>
#include "exceptionBase.h"

namespace iopc
{

/**
 * @brief Read/write lock synchronization primitive
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL RWLock {
public:
	RWLock()
		throw (IopcNoMemoryException, IopcRsrcLackException);
	~RWLock()
		throw ();
	/** @brief Waits for lock for reading. */
	void rlock()
		throw (IopcRsrcLackException, IopcDeadlockException);
	/** @brief Waits for lock for writing. */
	void wlock()
		throw (IopcDeadlockException);
	/** @brief Removes one lock. */
	void unlock()
		throw (IopcIncorrectUsageException);
private:	/* hiding */
	RWLock(const RWLock &);
	//RWLock& operator=(const RWLock &);
protected:
	pthread_rwlock_t rwlock;	/**< lock proper. */
};

}

#endif /*RWLOCK_H*/
