/**
 * @file oracle10gDatabaseImpl.cpp
 * 
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>) 22. - 23.2.2003
 * @author Petr Cermak
 */

#include "iopccommon/trace.h"
#include "oracle10gOci.h"
#include <string>
#include <iostream>
#include <sstream>
#include "iopcmeta/type.h"

using namespace std;

namespace iopc {

const pair<const string, const sb4> getOCIError(
	OCIEnv* ociEnvironment,
	OCIError* errorHandle,
	sword ociRetVal,
	const string& funcName,
	bool useEnvHandle)
{
	static sb4 errorCode = -1; 
	char buffer[512];
	
	switch (ociRetVal)
	{
		case OCI_SUCCESS : 
		case OCI_NO_DATA : 
			return pair<const string, const sb4>("", -1);
			break;

		case OCI_INVALID_HANDLE: 
			return pair<const string, const sb4>(funcName + " was passed an invalid handle (OCI_INVALID_HANDLE).", -1);
			break;											 

		case OCI_NEED_DATA:
			return pair<const string, const sb4>(funcName + " returned OCI_NEED_DATA.", -1);
			break;
			
		case OCI_STILL_EXECUTING:
			return pair<const string, const sb4>(funcName + " returned OCI_STILL_EXECUTING.", -1);
			break;

		case OCI_CONTINUE:
			return pair<const string, const sb4>(funcName + " returned OCI_CONTINUE.", -1);
			break;

		case OCI_SUCCESS_WITH_INFO:
		case OCI_ERROR:

			OCIErrorGet((useEnvHandle) ? (dvoid *)ociEnvironment : (dvoid *)errorHandle, 1, NULL, &errorCode, 
									(unsigned char *)buffer, sizeof(buffer), (useEnvHandle) ? OCI_HTYPE_ENV : OCI_HTYPE_ERROR);

			if (ociRetVal==OCI_ERROR) {
				return pair<const string, const sb4>(string("Error calling ") + funcName + ": " + buffer, errorCode);
			} else {
				return pair<const string, const sb4>(funcName + " additional info: " + buffer, errorCode);
			}
			
			break;

		default:
			ostringstream os;
			os << funcName << ": Unknown return value " << ociRetVal;
			return pair<const string, const sb4>(os.str(), -1);
			break;
	}
	IOPC_UNREACHABLE;
	return pair<const string, const sb4>("", -1);
}
/*
// DEPRECATED
ub2 getOracleType(const Type& varType)
{
	ub2 type=0; //Oracle external type of variable
	switch (varType.getDataType()) 
	{
		case Type::IOPC_TYPE_BOOL:	
												type = SQLT_INT;
												break;
		case Type::IOPC_TYPE_CHAR:
												type = SQLT_INT;
												break;
		case Type::IOPC_TYPE_UCHAR:
												type = SQLT_UIN;
												break;
		case Type::IOPC_TYPE_SCHAR:
												type = SQLT_INT;
												break;
		case Type::IOPC_TYPE_SHORT:
												type = SQLT_INT;			 
												break;
		case Type::IOPC_TYPE_USHORT:
												type = SQLT_UIN;			 
												break;
		case Type::IOPC_TYPE_INT:	
												type = SQLT_INT;			 
												break;
		case Type::IOPC_TYPE_UINT:	
												type = SQLT_INT;			 
												break;
		case Type::IOPC_TYPE_LONG:	
												type = SQLT_INT;
												break;
		
		case Type::IOPC_TYPE_REF:
		case Type::IOPC_TYPE_COMPLEX:
		case Type::IOPC_TYPE_ULONG:
												type = SQLT_UIN;
												break;

		case Type::IOPC_TYPE_FLOAT:
												type = SQLT_FLT;
												break;
		case Type::IOPC_TYPE_DOUBLE:
												type = SQLT_FLT;
												break;
		case Type::IOPC_TYPE_LONGDOUBLE:
													type = SQLT_FLT;
													break; 

		case Type::IOPC_TYPE_WCHART:
												type = SQLT_INT;
												break; 											
		case Type::IOPC_TYPE_STRING:
												type = SQLT_STR;
												break;

		case Type::IOPC_TYPE_WSTRING:
												type = SQLT_STR; //This is OK. After bind we say bind handle to accept Unicode...
												break;
		default:						type = UNKNOWN_DATA_TYPE;
	}
	return type;
}
*/

/*bool testik() {
	cout << "a";
	return true;
}*/

}
