/**
 * @file arcCacheBase.h
 *
 * @author Jan Hadrava
 * @date 2004
 * @ingroup iopclibcache
 */

#ifndef ARCCACHEBASE_H_
#define ARCCACHEBASE_H_


#include <map>

#include "extendedCache.h"
#include "arcCacheItem.h"
#include "iopccommon/defs.h"

namespace iopc {

class ArcCacheItem;

/** Common predecessor of extended object caches using ARC replacement policy,
* they use ArcCacheItem to represent stored objects
*
* @ingroup iopclibcache
*/
class DLL_EXTERNAL ArcCacheBase: public ExtendedCache {
friend class ArcCacheItem;
public:
	/** Initialize cache with the same strategies for all object.
	* \param minValue Minimal value of objects obliged to stay in the cache.
	* \param maxCost Maximal total cost of stored objects.
	* \param strat Default strategy.
	*/
	inline ArcCacheBase(long minValue = MINLONG, long maxCost = MAXLONG, const Strategy& strat = Strategy::defaultStrategy);
	/** Initialize cache with strategies passed by selector.
	* \param minValue Minimal value of objects obliged to stay in the cache.
	* \param maxCost Maximal total cost of stored objects.
	* \param strat strategy selector.
	**/
	inline ArcCacheBase(long minValue, long maxCost, StrategySelector* stratSel);
	inline virtual ~ArcCacheBase();
private:/*hiding*/
	ArcCacheBase(ArcCacheBase& );
	ArcCacheBase& operator=(ArcCacheBase& );
protected:
	ArcCacheBase(long minValue, long maxCost, StrategySelector* stratSel, const Strategy& strat);

	/** Adds a lock to an item containing an object
	* \param item The item the lock should be added to.
	* \param rdonly Type of the lock.
	* \return An item representing new lock (it's the same as the passed one)
	*/
	virtual ArcCacheItem& addLock(ArcCacheItem& item, bool rdonly) = 0;
	/** Removes one lock of an item representing an object
	* \param item The item the lock should be removed from.
	*/
	virtual void rmLock(ArcCacheItem &item) = 0;
};

inline ArcCacheBase::ArcCacheBase(long minValue, long maxCost, StrategySelector *stratSel)
: ExtendedCache(minValue, maxCost, stratSel)
{
}

inline ArcCacheBase::ArcCacheBase(long minValue, long maxCost, const Strategy &strat)
: ExtendedCache(minValue, maxCost, strat)
{
}

inline ArcCacheBase::ArcCacheBase(long minValue, long maxCost, StrategySelector* stratSel, const Strategy& strat)
: ExtendedCache(minValue, maxCost, stratSel, strat)
{
}

inline ArcCacheBase::~ArcCacheBase()
{
}

}/*namespace iopc*/

#endif /*ARCCACHEBASE_H_*/
