/**
 * @file strategySelector.h
 *
 * @author Michal Kipecky
 * @author Jan Hadrava
 * @ingroup iopclibcache
 */

#ifndef TYPES_H_
#define TYPES_H_

/******************************************************************************/
/*                                                                            */
/* POLiTe - Persistent Object Library Test                                    */
/*                                        Ph.D. Thesis by Mgr. Michal Kopecky */
/*                                                                            */
/* Charles University Prague                                                  */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* File name: ...                                                             */
/* Module: ......                                                             */
/*                                                                            */
/******************************************************************************/

namespace iopc {

// Chosen numbering system allows representing all strategies in one word.
// Each strategy occupies three bits
// This allows to simply mask out individual strategies

enum UpdatingStrategy {
	USDeferred,
	USImmed,
	USAsync
};

enum ReadingStrategy {
	RSCache,
	RSDatabase,
	RSTimestamp
};

enum LockingStrategy {           //LockingStrategy type definition
  LS_Default                 = 0x0008,
  LS_Current                 = 0x0010,
  LS_Inherited               = 0x0018,
  LS_None                    = 0x0020,
  LS_Shared                  = 0x0028,
  LS_Exclusive               = 0x0030
  };

#define LS_MASK 0x0038

enum WaitingStrategy {           //WaitingStrategy type definition
  WS_Default                 = 0x0040,
  WS_Current                 = 0x0080,
  WS_Inherited               = 0x00C0,
  WS_Wait                    = 0x0100,
  WS_Nowait                  = 0x0140
  };

#define WS_MASK 0x01C0

}/*namespace iopc*/


#endif /*TYPES_H_*/
