/**
 * @file cachedDatabase.h
 *
 * @ingroup iopclib
 *
 * @author Petr Cermak
 */

#ifndef CACHEDDATABASE_H_
#define CACHEDDATABASE_H_

#include "iopcdb/database.h"
#include "../cache/cache.h"

namespace iopc {


/**
 * @brief A @a Database class decorator that creates cached connections (see @a CachedConnection)
 *
 * To use it, pass a prototype of a @a Cache implementation to the constructor.
 * Consequent calls to @a getConnection() will clone the prototype,
 * create a new @a CachedConnection instance and pass this clone to it.
 */
class DLL_EXTERNAL CachedDatabase : public Database {

private:
	Database* inner;
	Cache* cache;

public:

	CachedDatabase(Database* database, Cache* cachePrototype);

	/**
	 * @brief Deletes the cache prototype and the decorated @a Database instance
	 */
	virtual ~CachedDatabase();

	virtual void commit();

	virtual void rollback();

	virtual Connection* getConnection(const std::string& connStr);

	virtual void returnConnection(Connection* connection);

	virtual void disconnect();

	virtual DatabaseImpl* getImplementation();

	virtual Database* getBasicDatabase();

	virtual Driver& getDriver();

}; // class


} // namespace


#endif /*CACHEDDATABASE_H_*/
