/**
 * @file attribute.h
 *
 * @ingroup iopcmeta
 *
 * @author Petr Cermak
 */

#ifndef ATTRIBUTE_H_
#define ATTRIBUTE_H_

#include "iopccommon/defs.h"
#include "iopccommon/metadataHolder.h"
#include <string>
#include <map>

namespace iopc {

class Type;

/**
 * @brief Represents an attribute of a class
 *
 * @ingroup iopcmeta
 *
 * Users are not supposed to create instances of this class
 */
class DLL_EXTERNAL Attribute : public MetadataHolder{
public:

	/**
	 * @brief Attribute access modifiers
	 */
	enum AttributeVisibility {
		VISIBILITY_PUBLIC,
		VISIBILITY_PROTECTED,
		VISIBILITY_PRIVATE
	};

	Attribute(const std::string& name, const Type& type, const AttributeVisibility visibility, const Type& owner);
	Attribute(const Attribute& src);
	Attribute(Attribute* const& src);

	/**
	 * @brief Returns the attribute name
	 */
	inline const std::string getName() const { return name; };

	/**
	 * @brief Returns data type of the attribute
	 */
	inline const Type& getType() const { return type; };

	/**
	 * @brief Returns the attribute access modifier
	 */
	inline const AttributeVisibility getVisibility() const { return visibility; };

	/**
	 * @brief Returns @a Type representing a class in which is the attribute defined
	 */
	inline const Type& getOwner() const { return owner; }


private:
	const Type& type;
	const std::string name;
	const AttributeVisibility visibility;
	const Type& owner;
};

}


#endif /*ATTRIBUTE_H_*/
