/**
 * @file enhancedTypeBase.cpp
 *
 * @ingroup iopcmeta
 *
 * @author Petr Cermak
 */

#include "enhancedTypeBase.h"
#include "iopccommon/trace.h"
#include "enhancedNumber.h"
#include "enhancedString.h"

namespace iopc {

void EnhancedTypeBase::setDirty(bool dirty)
{
	this->dirty = dirty;
	if (dirtyPtr != NULL && dirty) {
		*dirtyPtr = dirty;
	}
}

void EnhancedTypeBase::setDirtyPtr(bool* newDirtyPtr)
{
	dirtyPtr = newDirtyPtr;
	if (dirty) {
		*dirtyPtr = true;
	}
}

void EnhancedTypeBase::init()
{
	TypeDesc<EInt>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<int>::getType());
	TypeDesc<EUInt>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<unsigned int>::getType());
	TypeDesc<EChar>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<char>::getType());
	TypeDesc<EUChar>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<unsigned char>::getType());
	TypeDesc<ESChar>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<signed char>::getType());
	TypeDesc<EShort>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<short>::getType());
	TypeDesc<EUShort>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<unsigned short>::getType());
	TypeDesc<ELong>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<long>::getType());
	TypeDesc<EULong>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<unsigned long>::getType());
	TypeDesc<EFloat>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<float>::getType());
	TypeDesc<EDouble>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<double>::getType());
	TypeDesc<ELDouble>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<long double>::getType());
	TypeDesc<EWCharT>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<wchar_t>::getType());
	TypeDesc<EBool>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<bool>::getType());
	TypeDesc<EString>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<std::basic_string<char> >::getType());
	TypeDesc<EWString>::getType()["iopc.enhancedType.innerType"].setPointerValue((void*)&TypeDesc<std::basic_string<wchar_t> >::getType());
}

const Type& EnhancedTypeBase::getInnerTypeFor(const Type& enhancedType)
{
	// This is a little dirty implementation, but there is no other easy way
	// to statically determine inner type :(

	if (enhancedType.getDataTypeClass() != Type::IOPC_TYPECLASS_ENHANCED) {
		IOPC_LOG_THROW(IopcIncorrectUsageException("Enhanced type expected."));
	}

	const Type* type = (const Type*) enhancedType.getMeta("iopc.enhancedType.innerType").getPointerValue();
	return *type;
}




}
