#include "initTest.h"
#include "iopclib/types/scriptsGenerator.h"
#include "iopcdb/driverManager.h"
#include "iopcdb/driver.h"
#include "iopcdb/database.h"
#include "iopcdb/connection.h"
#include "iopccommon/trace.h"

#include <iostream>
#include <vector>

using namespace iopc;
using namespace std;

InitTest::InitTest(const string& testName)
 : TestCase(testName, true)
{
}

InitTest::~InitTest()
{
}


bool InitTest::doTest()
{
	Database* db = NULL;
	Connection* conn = NULL;
	bool ret = true;
	try {
		db = DriverManager::getDriver("IopcOracle10g").getDatabase(DBNAME);
		conn = db->getConnection(CONNSTR);
		conn->open();//s
		conn->setAutoCommit(false);

		vector<string> script;

		// drop script
		script = ScriptsGenerator::getDbDropScript(DriverManager::getDriver("IopcOracle10g"));

		for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
			//LogWriter::logInfo(*it);
			try {
				conn->sqlNonQuery(*it);
			} catch (std::exception& e) {
				LogWriter::logInfo(e.what());
			}
		}

		// create script
		script = ScriptsGenerator::getDbCreateScript(DriverManager::getDriver("IopcOracle10g"));

		for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
			conn->sqlNonQuery(*it);
			//LogWriter::logInfo(*it);
		}
	} catch (IopcException& e) {
		LogWriter::logError(e.what());
		ret = false;
	}

	if (db && conn) db->returnConnection(conn);
	if (db) DriverManager::getDriver("IopcOracle10g").returnDatabase(db);

	return ret;
}


static InitTest initTestInstance("Init");

