#include "iopccommon/trace.h"
#include "tests/persistenceTest.h"
#include "iopclib/dbPtr.h"
#include "iopcdb/driverManager.h"
#include "iopclib/types/dbTypeManager.h"
#include "iopclib/db/cachedDatabase.h"
#include "iopclib/db/cachedConnection.h"
#include "iopcdb/cursor.h"
#include "iopclib/cache/voidCache.h"
#include "iopclib/cache/lruCacheST.h"
#include "iopclib/cache/arcCacheST.h"

#include "testSchema/fakeType.h"
#include "testSchema/fakeObject.h"
#include "testSchema/testObject.h"
#include "testSchema/diamond.h"

using namespace iopc;
using namespace std;

PersistenceTest::PersistenceTest(const std::string& testName)
 : TestCase(testName)
{

}

PersistenceTest::~PersistenceTest()
{
}


bool PersistenceTest::doTest()
{
	Database* db = NULL;
	CachedConnection* conn = NULL;
	bool ret = true;
	try {
		db = new CachedDatabase(DriverManager::getDriver("IopcOracle10g").getDatabase(DBNAME), new HashArcCacheST());
		conn = (CachedConnection*)db->getConnection(CONNSTR);
		conn->open();
		conn->setAutoCommit(false);

		// prepare test

		IOPC_DEBUGLOG("1. DbPtr<FakeObject> objectPtr;");
		DbPtr<FakeObject> objectPtr;
		IOPC_DEBUGLOG("2. objectPtr->test();");
		objectPtr->test();
		IOPC_DEBUGLOG("3. objectPtr.bePersistent(conn);");
		objectPtr.bePersistent(conn);
		IOPC_DEBUGLOG("4. conn->commit();");
		conn->commit();
		IOPC_DEBUGLOG("5. --end--");

		DbPtr<DiamondClass4> objectPtr2;
		objectPtr2.bePersistent(conn);
		DbPtr<DiamondClass2> objectPtr3;
		objectPtr3.bePersistent(conn);
		DbPtr<DiamondClass2> objectPtr4;
		objectPtr4.bePersistent(conn);
		DbPtr<DiamondClass3> objectPtr5;
		objectPtr5.bePersistent(conn);
		DbPtr<DiamondClass6> objectPtr7;
		objectPtr7.bePersistent(conn);
		DbPtr<DiamondClass7> objectPtr8;
		objectPtr8.bePersistent(conn);

		DbPtr<DiamondClass5> objectPtr6;
		objectPtr6.bePersistent(conn);

		conn->commit();

		OidType oid = objectPtr6.getPersId().getOid();
		logMsg("objectPtr6->oid ", oid);
		const Type& t = DbTypeManager::getType(*conn, oid);
		if (t.isNot<DiamondClass5>()) {
			ret = false;
		}
		IOPC_DEBUGLOG("6. --load--");
		logMsg("objectPtr6->test6: " + objectPtr7->test6);
		logMsg("objectPtr6->test2: " + objectPtr7->test2);
		logMsg("objectPtr6->test1: " + objectPtr7->test1);
		if (objectPtr7->test6 != "value 6") {
			ret = false;
		}

		IOPC_DEBUGLOG("7. --load by oid--");
		DbPtr<DiamondClass5> loadedObject(conn, oid);
		logMsg("loadedObject->test5: " + loadedObject->test5);
		logMsg("loadedObject->test4: " + loadedObject->test4);
		logMsg("loadedObject->test3: " + loadedObject->test3);
		logMsg("loadedObject->test2: " + loadedObject->test2);
		logMsg("loadedObject->test1: " + loadedObject->test1);
		logMsg("loadedObject->oid: ", loadedObject->oid);
		logMsg("loadedObject->className: " + loadedObject->className);
		if (loadedObject->test3 != "value 3") {
			ret = false;
		}

		IOPC_DEBUGLOG("8. --update--");
		CachePtr<DiamondClass5> lockedObj = loadedObject.getCachePtr();
		SerialIdType serialId = lockedObj->serialId;
		lockedObj->test5 = "value 5.1";
		lockedObj->test2 = "value 2.1";
		//logMsg("MemCost :", (int) lockedObj->getMemCost());
		lockedObj.release();

		DbPtr<DiamondClass5> loadedObject2(conn, oid);
		if (loadedObject->test5 != "value 5.1" || loadedObject->test2 != "value 2.1"
			|| loadedObject->serialId != (serialId + 1)) {
			ret = false;
		}
		conn->getCache().updateDirty();
		IOPC_DEBUGLOG("9. --delete--");
		DbPtr<DiamondClass5> delobject(conn, oid);
		delobject.dbDelete();
		try {
			DbPtr<DiamondClass5> delobject2(conn, oid);
			logMsg("delbject->test2: " + delobject2->test1);
			ret = false;
		} catch (IopcException& e) {
			logMsg(e.what());
			ret = true;
		}

		IOPC_DEBUGLOG("10. --non oid-managed --");
		DbPtr<TestObject> testObject;
		testObject->attrInt = 1;
		testObject->attrDouble = 1.3;
		testObject->attrString = "TestObject";
		//logMsg("MemCost :", (int) testObject->getMemCost());
		testObject.bePersistent(conn);

		DbPtr<InheritedTestObject> inheritedObject;
		inheritedObject->attrInt = 2;
		inheritedObject->attrInt2 = 1002;
		inheritedObject->attrDouble = 1.2;
		inheritedObject->attrString = "InheritedTestObject";
		inheritedObject.bePersistent(conn);

		DbPtr<InheritedTestObject2> inheritedObject2;
		inheritedObject2->attrInt = 3;
		inheritedObject2->attrInt2 = 1003;
		inheritedObject2->attrInt3 = 2003;
		inheritedObject2->attrDouble = 1.3;
		inheritedObject2->attrString = "InheritedTestObject2";
		inheritedObject2.bePersistent(conn);

		KeyValues k1;
		k1.add("attrInt", 2);
		k1.add("attrString", "InheritedTestObject");
		DbPtr<InheritedTestObject> loadedTestObject1(conn, k1);
		logMsg(loadedTestObject1->toString());
		if (loadedTestObject1->attrInt2 != 1002 && loadedTestObject1->attrInt != 2 && loadedTestObject1->attrDouble != 1.2) {
			ret = false;
		}

		KeyValues k2;
		k2.add("attrInt", 3);
		k2.add("attrString", "InheritedTestObject2");
		DbPtr<InheritedTestObject2> loadedTestObject2(conn, k2);
		logMsg(loadedTestObject2->toString());
		if (loadedTestObject2->attrInt3 != 2003) {
			ret = false;
		}

		IOPC_DEBUGLOG("11. --Oracle OO Objects --");
		DbPtr<DiamondClass9> oo9;
		oo9->test9 = "testQ";
		oo9.bePersistent(conn);
		OidType oo9Oid = oo9.getPersId().getOid();
		conn->commit();
		conn->removeAll();

		DbPtr<DiamondClass9> oo9loaded(conn, oo9Oid);
		if (oo9loaded->test9 != "testQ") {
			ret = false;
		}
		oo9loaded->test9 = "testW";
		conn->commit();
		conn->removeAll();

		DbPtr<DiamondClass9> oo9loaded2(conn, oo9Oid);
		if (oo9loaded2->test9 != "testW") {
			ret = false;
		}
		oo9loaded2.dbDelete();
		conn->commit();
		conn->removeAll();
		try {
			DbPtr<DiamondClass9> delobject3(conn, oo9Oid);
			logMsg("delobject3->test9: " + delobject3->test9);
			ret = false;
		} catch (IopcException& e) {
			logMsg(e.what());
			ret = true;
		}

		DbPtr<DiamondClass8> d8;
		d8.bePersistent(conn);


	} catch (IopcException& e) {
		LogWriter::logError(e.what());
		ret = false;
	}

	if (db && conn) db->returnConnection(conn);
	if (db) DriverManager::getDriver("IopcOracle10g").returnDatabase(db);

	return ret;
}

static PersistenceTest PersistenceTestInstance("Persistence Test");
